@@ -1,21 +1,19 @@
 package Yote::WebAppServer;
 
-#
-# Proof of concept server with main loop.
-#
 use strict;
 
 use forks;
 use forks::shared;
 
 use CGI;
+use IO::Handle;
 use Net::Server::HTTP;
 use MIME::Base64;
 use JSON;
-use CGI;
 use Data::Dumper;
 
 use Yote::AppRoot;
+use Yote::FileHelper;
 use Yote::ObjProvider;
 
 use base qw(Net::Server::HTTP);
@@ -24,11 +22,15 @@ use vars qw($VERSION);
 $VERSION = '0.081';
 
 
-my( @commands, %prid2wait, %prid2result, $singleton, @saves );
+my( @commands, %prid2wait, %prid2result, $singleton );
 share( @commands );
 share( %prid2wait );
 share( %prid2result );
-share( @saves );
+
+
+# ------------------------------------------------------------------------------------------
+#      * INIT METHODS *
+# ------------------------------------------------------------------------------------------
 
 sub new {
     my $pkg = shift;
@@ -37,47 +39,6 @@ sub new {
     return $singleton;
 }
 
-sub start_server {
-    my( $self, @args ) = @_;
-    my $args = scalar(@args) == 1 ? $args[0] : { @args };
-    $self->{args} = $args;
-    $self->{args}{webroot} ||= '/usr/local/yote/html';
-
-    Yote::ObjProvider::init( %$args );
-
-    # fork out for three starting threads
-    #   - one a multi forking server (parent class)
-    #   - one for a cron daemon inside of Yote.
-    #   - and the parent thread an event loop.
-
-    # cron thread
-    my $root = Yote::YoteRoot::fetch_root();
-    my $cron = $root->get__crond();
-    my $cron_thread = threads->new( sub { $self->_crond( $cron->{ID} ); } );
-    $self->{cron_thread} = $cron_thread;
-
-    # server thread
-    my $server_thread = threads->new( sub { $self->run( %$args ); } );
-    $self->{server_thread} = $server_thread;
-
-    _poll_commands();
-
-    $server_thread->join;
-
-   Yote::ObjProvider::disconnect();
-
-} #start_server
-
-sub shutdown {
-    my $self = shift;
-    print STDERR "Shutting down yote server \n";
-    Yote::ObjProvider::stow_all();
-    print STDERR "Killing threads \n";
-    $self->{server_thread}->detach();
-    $self->{saving_thread}->detach();
-    print STDERR "Shut down server thread.\n";
-} #shutdown
-
 #
 # Sets up Initial database server and tables.
 #
@@ -86,6 +47,16 @@ sub init_server {
    Yote::ObjProvider::init_datastore( @args );
 } #init_server
 
+# ------------------------------------------------------------------------------------------
+#      * PUBLIC METHODS *
+# ------------------------------------------------------------------------------------------
+
+
+sub do404 {
+    my $self = shift;
+    $self->send_status( "404 Not Found" );
+    print "Content-Type: text/html\n\nERROR : 404\n";
+}
 
 #
 # Called when a request is made. This does an initial parsing and
@@ -100,6 +71,12 @@ sub init_server {
 sub process_http_request {
     my $self = shift;
 
+    my $content_length = $ENV{CONTENT_LENGTH};
+    if( $content_length > 5_000_000 ) { #make this into a configurable field
+	$self->do404();
+	return;
+    }
+
     #
     # There are two requests :
     #   * web page
@@ -112,18 +89,31 @@ sub process_http_request {
     #   * d  - data
     #   * oi - object id to invoke command on
     #   * p  - ip address
-    #   * t  - token for verification
+    #   * t  - login token for verification
+    #   * at - app (non-login) token for verification
     #   * w  - if true, waits for command to be processed before returning
     #
-    my $CGI  = new CGI;
-    my $vars = $CGI->Vars();
 
-    my( $uri, $remote_ip, $verb ) = @ENV{'PATH_INFO','REMOTE_ADDR','REQUEST_METHOD'};
+    my( $uri, $remote_ip ) = @ENV{'PATH_INFO','REMOTE_ADDR'};
+
+    print STDERR Data::Dumper->Dump(["REQUEST FOR $uri"]);
 
     $uri =~ s/\s+HTTP\S+\s*$//;
-#    print STDERR ")STaRt pid $$ : $verb $uri : ";
+
     my( @path ) = grep { $_ ne '' && $_ ne '..' } split( /\//, $uri );
-    if( $path[0] eq '_' ) {
+    print STDERR Data::Dumper->Dump(["PATH : '$path[0]'"]);
+    if( $path[0] eq '_' || $path[0] eq '_u' ) { # _ is normal yote io, _u is upload file
+	my( $vars, $return_header );
+
+	if( $path[0] eq '_' ) {
+	    my $CGI  = new CGI;
+	    $vars = $CGI->Vars();
+	    $return_header = "Content-Type: text/json\n\n";
+	}
+	else {
+	    $vars = Yote::FileHelper->__ingest();
+	    $return_header = "Content-Type: text/html\n\n";
+	}
 
         my $action = pop( @path );
         my $obj_id = int( pop( @path ) ) || 1;
@@ -137,6 +127,7 @@ sub process_http_request {
             oi => $obj_id,
             p  => $remote_ip,
             t  => $vars->{t},
+	    at => $vars->{at},
             w  => $wait,
         };
 
@@ -181,8 +172,9 @@ sub process_http_request {
                 $result = $prid2result{$procid};
                 delete $prid2result{$procid};
             }
-#            print STDERR "Sending result $result\n";
-            print "Content-Type: text/json\n\n";
+            print STDERR "Sending result $return_header $result\n";
+
+	    print $return_header;
             print "$result";
         }
         else {  #not waiting for an answer, but give an acknowledgement
@@ -197,11 +189,16 @@ sub process_http_request {
 	if( -d "<$root/$dest" ) {
 	    $dest .= '/index.html';
 	}
-	print STDERR Data::Dumper->Dump(["$root/$dest","SERV"]);
 	if( open( IN, "<$root/$dest" ) ) {
-	    if( $dest =~ /^yote\/js/ ) {
+	    if( $dest =~ /\.js$/i ) {
 		print "Content-Type: text/javascript\n\n";
 	    }
+	    elsif( $dest =~ /\.css$/i ) {
+		print "Content-Type: text/css\n\n";
+	    }
+	    elsif( $dest =~ /\.(jpg|gif|png|jpeg)$/i ) {
+		print "Content-Type: image/$1\n\n";
+	    }
 	    else {
 		print "Content-Type: text/html\n\n";
 	    }
@@ -210,14 +207,102 @@ sub process_http_request {
             }
             close( IN );
 	} else {
-	    do404();
+	    $self->do404();
 	}
-#        print STDERR "<END---------------- PROC REQ $$ ------------------>\n";
 	return;
     } #serve html
 
 } #process_request
 
+
+sub shutdown {
+    my $self = shift;
+    print STDERR "Shutting down yote server \n";
+    Yote::ObjProvider::stow_all();
+    print STDERR "Killing threads \n";
+    $self->{server_thread}->detach();
+    $self->{saving_thread}->detach();
+    print STDERR "Shut down server thread.\n";
+} #shutdown
+
+sub start_server {
+    my( $self, @args ) = @_;
+    my $args = scalar(@args) == 1 ? $args[0] : { @args };
+    $self->{args} = $args;
+    $self->{args}{webroot} ||= '/usr/local/yote/html';
+    $self->{args}{upload} ||= '/usr/local/yote/html/upload';
+
+    Yote::ObjProvider::init( %$args );
+
+    # fork out for three starting threads
+    #   - one a multi forking server (parent class)
+    #   - one for a cron daemon inside of Yote. (PENDING)
+    #   - and the parent thread an event loop.
+
+    my $root = Yote::YoteRoot::fetch_root();
+
+    # @TODO - finish the cron and uncomment this
+    # cron thread
+    #my $cron = $root->get__crond();
+    #my $cron_thread = threads->new( sub { $self->_crond( $cron->{ID} ); } );
+    #$self->{cron_thread} = $cron_thread;
+
+    # make sure the filehelper knows where the data directory is
+    $Yote::WebAppServer::YOTE_ROOT_DIR = $self->{args}{root_dir};
+    $Yote::WebAppServer::DATA_DIR      = $self->{args}{data_dir};
+    $Yote::WebAppServer::FILE_DIR      = $self->{args}{data_dir} . '/holding';
+    $Yote::WebAppServer::WEB_DIR       = $self->{args}{webroot};
+    $Yote::WebAppServer::UPLOAD_DIR    = $self->{args}{webroot}. '/uploads';
+    mkdir( $Yote::WebAppServer::DATA_DIR );
+    mkdir( $Yote::WebAppServer::FILE_DIR );
+    mkdir( $Yote::WebAppServer::WEB_DIR );
+    mkdir( $Yote::WebAppServer::UPLOAD_DIR );
+
+    # update @INC library list
+    my $paths = $root->get__application_lib_directories([]);
+    push @INC, @$paths;
+
+    # server thread
+    my $server_thread = threads->new( sub { $self->run( %$args ); } );
+    $self->{server_thread} = $server_thread;
+
+    _poll_commands();
+
+    $server_thread->join;
+
+   Yote::ObjProvider::disconnect();
+
+} #start_server
+
+
+
+# ------------------------------------------------------------------------------------------
+#      * PRIVATE METHODS *
+# ------------------------------------------------------------------------------------------
+
+
+sub _crond {
+    my( $self, $cron_id ) = @_;
+
+    while( 1 ) {
+	sleep( 60 );
+	{
+	    lock( @commands );
+	    push( @commands, [ {
+		a  => 'check',
+		ai => 1,
+		d  => 'eyJkIjoxfQ==',
+		oi => $cron_id,
+		p  => undef,
+		t  => undef,
+		w  => 0,
+			       }, $$] );
+	    cond_broadcast( @commands );
+	}
+    } #infinite loop
+
+} #_crond
+
 #
 # Run by a thread that constantly polls for commands.
 #
@@ -245,9 +330,8 @@ sub _poll_commands {
 } #_poll_commands
 
 sub _process_command {
-    my $req = shift;
+    my( $req ) = @_;
     my( $command, $procid ) = @$req;
-#    print STDERR Data::Dumper->Dump([$command,"CMD"]);
     my $wait = $command->{w};
 
     my $resp;
@@ -256,37 +340,54 @@ sub _process_command {
         my $obj_id = $command->{oi};
         my $app_id = $command->{ai};
 
-        my $app        = Yote::ObjProvider::fetch( $app_id ) || Yote::YoteRoot::fetch_root();
+        my $app         = Yote::ObjProvider::fetch( $app_id ) || Yote::YoteRoot::fetch_root();
+
+        my $data        = _translate_data( from_json( MIME::Base64::decode( $command->{d} ) )->{d} );
+        my $login       = $app->token_login( $command->{t}, undef, $command->{p} );
+	my $guest_token = $command->{gt};
 
-        my $data       = _translate_data( from_json( MIME::Base64::decode( $command->{d} ) )->{d} );
-        my $login = $app->token_login( { t => $command->{t}, _ip => $command->{p} } );
 	print STDERR Data::Dumper->Dump(["INCOMING",$data,$command,$login]);
 
 
-        my $app_object =Yote::ObjProvider::fetch( $obj_id ) || $app;
+        my $app_object = Yote::ObjProvider::fetch( $obj_id ) || $app;
         my $action     = $command->{a};
         my $account;
-
-        # hidden parts of the args
-        if( ref( $data ) eq 'HASH' ) {
-            $data->{_ip} = $command->{p};
-        }
-
         if( $login ) {
-            $account = $app->_get_account( $login );
-
-            if( ! $app->_account_can_access( $account, $app_object ) ) {
-                die "Access Error";
-            }
+            $account = $app->__get_account( $login );
         }
 	Yote::ObjProvider::reset_changed();
 
-#	print STDERR Data::Dumper->Dump(["doing $action on ", $app_object, 'with data',$data,"and account",$account,'and login',$account?$account->get_login():'none'] );
-        my $ret = $app_object->$action( $data, $account );
+        my $ret = $app_object->$action( $data, $account, $command->{p} );
 
-	my $dirty_delta = Yote::ObjProvider::fetch_changed();
+	my $dirty_delta = Yote::ObjProvider::__fetch_changed();
 
-        $resp = { r => $app_object->_obj_to_response( $ret, $account, 1 ), d => $dirty_delta };
+	my( $dirty_data );
+	if( @$dirty_delta ) {
+	    $dirty_data = {};
+	    my( @allowed_dirty );
+	    if( $login && $Yote::ObjProvider::LOGIN_OBJECTS->{ $login->{ID} } ) {
+		@allowed_dirty = grep { $Yote::ObjProvider::LOGIN_OBJECTS->{ $login->{ID} }{ $_ } } @$dirty_delta;
+	    } else {
+		@allowed_dirty = grep { $Yote::ObjProvider::GUEST_TOKEN_OBJECTS->{ $guest_token }{ $_ } } @$dirty_delta;
+	    }
+	    for my $d_id ( @allowed_dirty) {
+		my $dobj = Yote::ObjProvider::fetch( $d_id );
+		if( ref( $dobj ) eq 'ARRAY' ) {
+		    $dirty_data->{$d_id} = { map { $_ => Yote::ObjProvider::xform_in( $dobj->[$_] ) } (0..$#$dobj) };
+		} elsif( ref( $dobj ) eq 'HASH' ) {
+		    $dirty_data->{$d_id} = { map { $_ => Yote::ObjProvider::xform_in( $dobj->{ $_ } ) } keys %$dobj };
+		} else {
+		    $dirty_data->{$d_id} = { map { $_ => $dobj->{DATA}{$_} } grep { $_ !~ /^_/ } keys %{$dobj->{DATA}} };
+		}
+	    }
+	} #if there was a dirty delta
+
+        $resp = $dirty_data ? { r => $app_object->__obj_to_response( $ret, $login, 1, $guest_token ), d => $dirty_data } : { r => $app_object->__obj_to_response( $ret, $login, 1, $guest_token ) };
+
+	if( $login ) {
+	    $Yote::ObjProvider::LOGIN_OBJECTS->{ $login->{ID} }{ $app_object->{ID} } = 1;
+	}
+	
     };
     if( $@ ) {
 	my $err = $@;
@@ -314,43 +415,31 @@ sub _process_command {
 
 } #_process_command
 
-sub _crond {
-    my( $self, $cron_id ) = @_;
-
-    while( 1 ) {
-	sleep( 60 );
-	{
-	    lock( @commands );
-	    push( @commands, [ { 
-		a  => 'check',	    
-		ai => 1,
-		d  => 'eyJkIjoxfQ==',
-		oi => $cron_id,
-		p  => undef,
-		t  => undef,
-		w  => 0,
-			       }, $$] );
-	    cond_broadcast( @commands );
-	}
-    } #infinite loop
-    
-} #_crond
-
 #
 # Translates from vValue and reference_id to values and references
 #
 sub _translate_data {
-    my $val = shift;
+    my( $val ) = @_;
 
     if( ref( $val ) ) { #from javacript object, or hash. no fields starting with underscores accepted
         return { map {  $_ => _translate_data( $val->{$_} ) } grep { index( $_, '_' ) != 0 } keys %$val };
     }
     return undef unless $val;
-    return index($val,'v') == 0 ? substr( $val, 1 ) : Yote::ObjProvider::fetch( $val );
-}
+    if( index($val,'v') == 0 ) {
+	return substr( $val, 1 );
+    }
+    elsif( index($val,'u') == 0 ) {  #file upload contains an encoded hash
+	my $filestruct   = from_json( substr( $val, 1 ) );
 
-sub do404 {
-    print "Content-Type: text/html\n\nERROR : 404\n";
+	my $filehelper = new Yote::FileHelper();
+	$filehelper->set_content_type( $filestruct->{content_type} );
+	$filehelper->__accept( $filestruct->{filename} );
+
+	return $filehelper;
+    }
+    else {
+	return Yote::ObjProvider::fetch( $val );
+    }
 }
 
 1;
@@ -376,9 +465,32 @@ Additional parameters are passed to the datastore.
 
 The server set up uses Net::Server::Fork receiving and sending messages on multiple threads. These threads queue up the messages for a single threaded event loop to make things thread safe. Incomming requests can either wait for their message to be processed or return immediately.
 
+=head1 PUBLIC METHODS
+
+=over 4
+
+=item do404
+
+Return a 404 not found page and exit.
+
+=item process_http_request( )
+
+This implements Net::Server::HTTP and is called automatically for each incomming request.
+
+=item shutdown( )
+
+Shuts down the yote server, saving all unsaved items.
+
+=item start_server( )
+
+=back
+
 =head1 BUGS
 
-There are likely bugs to be discovered. This is alpha software
+There are likely bugs to be discovered. This is alpha software.
+
+More important than bugs are the incompletions, such as the cron not being taken offline
+for now, and the Cache not completed. These are top todo at the time of writing.
 
 =head1 AUTHOR
 
